<?php

namespace Tablesome\Includes\Modules;

if (!class_exists('\Tablesome\Includes\Modules\Migrate_Old_Crud_Data_To_TablesomeDB')) {
    class Migrate_Old_Crud_Data_To_TablesomeDB
    {
        public function run()
        {
            $tablesomeDB = new \Tablesome\Includes\Modules\TablesomeDB\TablesomeDB();
            $crud = new \Tablesome\Includes\Db\CRUD();

            /** Get all tablesome posts including trash posts */
            $tables = get_posts(array(
                'post_type' => TABLESOME_CPT,
                'post_status' => 'publish,trash',
                'numberposts' => -1,
            ));

            foreach ($tables as $table) {
                /** Get Table Metadata  */
                $meta_data = get_post_meta($table->ID, 'tablesome_data', true);

                /** Can't create a tablesomeDB table without meta_data */
                if (empty($meta_data)) {
                    continue;
                }
                /** Create a New Table */
                $table_instance = $tablesomeDB->create_table_instance($table->ID, $meta_data);

                /*** Get all records from DB({wp_prefix}_tablesome_records) */
                $rows = $crud->get_all_rows($table->ID, array()); /*** Continue the loop if the rows is empty */
                if (empty($rows)) {
                    continue;
                }
                $records = array();
                foreach ($rows as $row) {
                    $records[] = array(
                        'post_id' => $row->post_id,
                        'author_id' => $row->author_id,
                        'created_at' => $row->created_at,
                        'updated_at' => $row->updated_at,
                        'content' => json_decode($row->content, true),
                    );
                }

                $insert_info = $tablesomeDB->insert_many($table->ID, $meta_data, $records);

                $table_records_count = $table_instance->count();
                $records_inserted_count = isset($insert_info['records_inserted_count']) ? $insert_info['records_inserted_count'] : 0;

                if ($table_records_count == $records_inserted_count) {
                    error_log('**** Table ' . $table->ID . ' - migration done..  *****');

                } else {
                    error_log('**** Table ' . $table->ID . ' - migration failed.. *****');
                }
                error_log('[Migrated Records Count ] : ' . $records_inserted_count);
            }

        }
    }
}