<?php

namespace Tablesome\Includes\Modules\TablesomeDB_Rest_Api;

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('\Tablesome\Includes\Modules\TablesomeDB_Rest_Api\Import')) {
    class Import
    {
        public function import_records($request)
        {
            $crud = new \Tablesome\Includes\Db\CRUD();
            error_log('[START] : ' . get_app_memory_usage());
            $params = $request->get_params();

            $table_id = isset($params['table_id']) ? $params['table_id'] : 0;

            if (empty($table_id)) {
                $error_code = "REQUIRED_POST_ID";
                return new \WP_Error($error_code, $this->get_error_message($error_code));
            }

            $post = get_post($table_id);

            if (empty($post) || $post->post_type != TABLESOME_CPT) {
                $error_code = "INVALID_POST";
                return new \WP_Error($error_code, $this->get_error_message($error_code));
            }

            $meta_data = get_tablesome_data($table_id);
            $columns = $meta_data['columns'];
            $props = [
                'columns' => [],
                'rows_count' => 0,
            ];

            $current_batch_no = 1;
            $record_counter = 0;

            $records = isset($params['records_inserted']) ? $params['records_inserted'] : [];
            foreach ($records as $index => $record) {
                $content = $this->get_row_cells($record, $columns);

                $props["rows"][] = array(
                    'content' => $content,
                    'rank_order' => $record['rank_order'],
                );

                $end_row_index = ($current_batch_no * TABLESOME_BATCH_SIZE) - 1;
                if ($index == $end_row_index) {
                    $current_batch_no++;
                    $crud->bulk_inserting($table_id, $props);
                    unset($props['rows']);
                }

                $record_counter++;
                if ($record_counter == TABLESOME_MAX_RECORDS_TO_READ) {
                    break;
                }
            }

            if (isset($props["rows"]) && !empty($props["rows"]) && $record_counter < $end_row_index) {
                $crud->bulk_inserting($table_id, $props);
                unset($props['rows']);
            }

            $props["rows_count"] = $record_counter;

            $message = 'No records inserts';
            if ($record_counter > 0) {
                $message = $record_counter . ' records inserted successfully';
            }

            $response_data = array(
                'inserts_records_count' => $record_counter,
                'message' => $message,
            );
            error_log('[END] : ' . get_app_memory_usage());
            return rest_ensure_response($response_data);
        }

        public function get_row_cells($record, $columns)
        {
            $cells = array();

            foreach ($record['content'] as $index => $cell_value) {
                $column_id = $columns[$index]['id'];
                $cells[$column_id] = $cell_value;
            }
            return $cells;
        }
    }
}