<?php

namespace Tablesome\Includes\Tracking;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

if (!class_exists('\Tablesome\Includes\Tracking\Model')) {
    class Model
    {
        public function get_data()
        {
            $crud = new \Tablesome\Includes\Db\CRUD();
            $options_data = $this->get_options_data();
            $data = $this->get_required_fields_data_from_setting($options_data);
            $args = $this->get_exclude_tables_args();
            $table_collection_data = $crud->get_tables_collection($args);
            $data = array_merge($data, $table_collection_data);
            return $data;
        }

        public function get_exclude_tables_args()
        {
            $option_name = TABLESOME_SAMPLE_TABLE_OPTION;
            $sample_table_id = \get_option($option_name);

            if (empty($sample_table_id)) {
                return [];
            }
            return array(
                'post__not_in' => array($sample_table_id),
            );
        }

        public function get_options_data()
        {
            $option_name = TABLESOME_OPTIONS;
            return \get_option($option_name);
        }

        public function get_required_fields()
        {
            return array(
                "num_of_records_per_page",
                "search",
                "hide_table_header",
                "sorting",
                "filters",
                "mobile_layout_mode",
                "style_disable",
                "min_column_width",
            );
        }

        public function get_required_fields_data_from_setting($data)
        {
            $required_fields = $this->get_required_fields();

            foreach ($data as $key => $value) {
                if (!in_array($key, $required_fields)) {
                    unset($data[$key]);
                }
            }
            return $data;
        }

        public function get_user_props()
        {
            $fs_utils = new \Tablesome\Includes\Freemius_Utils();
            $fs_collection_props = $fs_utils->get_collection_props();
            $props = array();
            $props['plan'] = $fs_collection_props['plan'];
            $props['wp_version'] = $fs_collection_props['wp_version'];
            $props['php_version'] = $fs_collection_props['php_version'];
            $props['site_url'] = $fs_collection_props['site_url'];
            $props['email'] = $fs_collection_props['email'];
            $current_user = wp_get_current_user();
            if (isset($current_user) && !empty($current_user)) {
                $data = isset($current_user->data) ? $current_user->data : null;
                $props['login_name'] = isset($data->user_login) ? $data->user_login : '';
                $props['role'] = isset($current_user->roles[0]) ? $current_user->roles[0] : '';
            }
            return $props;
        }

        public function get_browser_name($user_agent)
        {
            if (strpos($user_agent, 'Opera') || strpos($user_agent, 'OPR/')) {
                return 'Opera';
            } elseif (strpos($user_agent, 'Edge')) {
                return 'Edge';
            } elseif (strpos($user_agent, 'Chrome')) {
                return 'Chrome';
            } elseif (strpos($user_agent, 'Safari')) {
                return 'Safari';
            } elseif (strpos($user_agent, 'Firefox')) {
                return 'Firefox';
            } elseif (strpos($user_agent, 'MSIE') || strpos($user_agent, 'Trident/7')) {
                return 'Internet Explorer';
            }

            return 'Other';
        }
    }
}